<?php

namespace Drupal\piwik_pro\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Piwik Pro settings for this site.
 */
class PiwikProAdminSettingsForm extends ConfigFormBase {

  /**
   * The constructor method.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'piwik_pro_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['piwik_pro.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('piwik_pro.settings');

    $form['general'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('General settings'),
    ];
    $form['general']['piwik_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Container address (URL)'),
      '#description' => $this->t('The account address with <strong>containers</strong> added to the address.<br>
F.e.: <code>https://yourname.<strong>containers</strong>.piwik.pro/</code> or
<code>https://yourname.piwik.pro/<strong>containers</strong></code>. Always end with a slash (/).'),
      '#default_value' => $config->get('piwik_domain'),
    ];
    $form['general']['site_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site ID'),
      '#description' => $this->t('The ID for your site in Piwik Pro. <a href=":piwik_help_url" target="_blank">Where to find it?</a>', [':piwik_help_url' => 'https://help.piwik.pro/support/questions/find-website-id/']),
      '#default_value' => $config->get('site_id'),
    ];
    $form['general']['sync_snippet'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Additional container (sync)'),
      '#description' => $this->t('Add this container if you want to use sync tags. It loads tags before the page content loads.'),
      '#default_value' => $config->get('sync_snippet'),
    ];
    $form['general']['data_layer'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Data layer'),
      '#description' => $this->t('Default: <code>dataLayer</code>.
Rename the data layer if you use other data layers to prevent interference.
<a href=":piwik_help_datalayer" target="_blank">How to check it?</a>', [':piwik_help_datalayer' => 'https://developers.piwik.pro/en/latest/tag_manager/data_layer_name.html#data-layer-name-guidelines']),
      '#default_value' => $config->get('data_layer'),
    ];

    $form['tracking'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('What to track'),
    ];
    $form['tracking']['piwik_pro_visibility_request_path_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add tracking to specific pages'),
      '#options' => [
        $this->t('Every page except the listed pages'),
        $this->t('The listed pages only'),
        ],
      '#default_value' => $config->get('visibility.request_path_mode'),
    ];
    $visibility_request_path_pages = $config->get('visibility.request_path_pages');
    $form['tracking']['piwik_pro_visibility_request_path_pages'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Pages'),
      '#title_display' => 'invisible',
      '#default_value' => !empty($visibility_request_path_pages) ? $visibility_request_path_pages : '',
      '#description' => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", ['%blog' => '/blog', '%blog-wildcard' => '/blog/*', '%front' => '<front>']),
      '#rows' => 10,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // Validate URL.
    $value = rtrim($form_state->getValue('piwik_domain'), '/');
    if (!filter_var( $value, FILTER_VALIDATE_URL)) {
      $form_state->setErrorByName('piwik_domain', $this->t('Wrong <code>Container address (URL)</code> value.'));
    }

    // Validate Site ID.
    $value = strtolower($form_state->getValue('site_id'));
    if (preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $value ) !== 1) {
      $form_state->setErrorByName('site_id', $this->t('Wrong <code>Site ID</code> value.'));
    }

    // Validate data-layers.
    if (preg_match('/^[a-zA-Z_$][0-9a-zA-Z_$]*$/', $form_state->getValue('data_layer')) !== 1) {
      $form_state->setErrorByName('data_layer', 'Wrong <code>Data layer</code> value.' );
    }

    // Trim some text values.
    $form_state->setValue('piwik_pro_visibility_request_path_pages', trim($form_state->getValue('piwik_pro_visibility_request_path_pages')));

    // Verify that every path is prefixed with a slash.
    if (!empty($form_state->getValue('piwik_pro_visibility_request_path_pages'))) {
      $pages = preg_split('/(\r\n?|\n)/', $form_state->getValue('piwik_pro_visibility_request_path_pages'));
      foreach ($pages as $page) {
        if (strpos($page, '/') !== 0 && $page !== '<front>') {
          $form_state->setErrorByName('piwik_pro_visibility_request_path_pages', $this->t('Path "@page" not prefixed with slash.', ['@page' => $page]));
          break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('piwik_pro.settings');

    // The url should always end with a slash.
    $piwik_domain = $form_state->getValue('piwik_domain');
    if (!(substr($piwik_domain, -1) ==="/")) {
      $piwik_domain .= '/';
    }

    $config
      ->set('site_id', $form_state->getValue('site_id'))
      ->set('piwik_domain', $piwik_domain)
      ->set('sync_snippet', $form_state->getValue('sync_snippet'))
      ->set('data_layer', $form_state->getValue('data_layer'))
      ->set('visibility.request_path_mode', $form_state->getValue('piwik_pro_visibility_request_path_mode'))
      ->set('visibility.request_path_pages', $form_state->getValue('piwik_pro_visibility_request_path_pages'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
